<?php

class Dap_UserCertificate {
	var $id;
	var $user_id;
	var $course_id;	
	var $date;
	
	function getId() {
		return $this->id;
	}
	function setId($o) {
		$this->id = $o;
	}
	
	function getUserId() {
		return $this->user_id;
	}
	function setUserId($o) {
		$this->user_id = $o;
	}
	
	function getCourseId() {
		return $this->course_id;
	}
	function setCourseId($o) {
		$this->course_id = $o;
	}
	
	
	function getDate() {
		return $this->date;
	}
	
	function setDate($o) {
		$this->date = $o;
	}
	
	public function create() {
		try {
			$dap_dbh = Dap_Connection::getConnection();
			$sql = "insert into dap_user_certificate
						( course_id, user_id, date )
					values
						( :course_id, :user_id, :date)";
			$stmt = $dap_dbh->prepare($sql);			
			$stmt->bindParam(':course_id', $this->getCourseId(), PDO::PARAM_INT);
			$stmt->bindParam(':user_id', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':date', $this->getDate(), PDO::PARAM_STR);
			
			$stmt->execute();			
			$last_id = $dap_dbh->lastInsertId();
			$stmt = null;
			$dap_dbh = null;
			return $last_id;

		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	
	public function update() {
		try {
			$dap_dbh = Dap_Connection::getConnection();
			
			$sql = "update dap_user_certificate set
					course_id = :course_id,
					user_id = :user_id,					
					date = :date					
					where
						id = :id
					";
					
			$stmt = $dap_dbh->prepare($sql);			 
			$stmt->bindParam(':course_id', $this->getCourseId(), PDO::PARAM_INT);
			$stmt->bindParam(':user_id', $this->getUserId(), PDO::PARAM_INT);
			$stmt->bindParam(':date', $this->getDate(), PDO::PARAM_STR);
 			$stmt->bindParam(':id', $this->getId(), PDO::PARAM_INT); 			 
			$stmt->execute();
			
			$stmt->execute();
			$stmt = null;
			$dap_dbh = null;

			return;

		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	
	public static function loadById($id) {
		$dap_dbh = Dap_Connection::getConnection();
		$sqbdataReturn = null;

		//Load product details from database
		$sql = "select *
			from
				dap_user_certificate
			where
				id = :id";

		$stmt = $dap_dbh->prepare($sql);
		$stmt->bindParam(':id', $id, PDO::PARAM_STR);
		$stmt->execute();
			while ($row = $stmt->fetch()) {
				$sqbdata = new Dap_UserCertificate();					
				$sqbdata->setId( $row["id"] );				
				$sqbdata->setCourseId( stripslashes($row["course_id"]) );
				$sqbdata->setUserId( stripslashes($row["user_id"]) );				
				$sqbdata->setDate( stripslashes($row["date"]) );				
				$sqbdataReturn = $sqbdata;
		}

		return $sqbdataReturn;
	}
	 
	 
	public static function loadByUserId($userId) {
		$dap_dbh = Dap_Connection::getConnection();
		$sqbdataReturn = false;

		//Load product details from database
		$sql = "select *
			from
				dap_user_certificate
			where
				user_id = :user_id";

		$stmt = $dap_dbh->prepare($sql);
		$stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);
		$stmt->execute();
			while ($row = $stmt->fetch()) {
				$sqbdata = new Dap_UserCertificate();					
				$sqbdata->setId( $row["id"] );				
				$sqbdata->setCourseId( stripslashes($row["course_id"]) );
				$sqbdata->setUserId( stripslashes($row["user_id"]) );				
				$sqbdata->setDate( stripslashes($row["date"]) );				
				$sqbdataReturn[] = $sqbdata;
		}

		return $sqbdataReturn;
	} 
	
	public static function loadByCourseIdAndUserId($courseId ,$userId) {
		$dap_dbh = Dap_Connection::getConnection();
		$sqbdataReturn = false;

		//Load product details from database
		$sql = "select *
			from
				dap_user_certificate
			where
				user_id = :user_id AND course_id = :course_id";

		$stmt = $dap_dbh->prepare($sql);
		$stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);
		$stmt->bindParam(':course_id', $courseId, PDO::PARAM_INT);
		$stmt->execute();
			while ($row = $stmt->fetch()) {
				
				$sqbdata = new Dap_UserCertificate();					
				$sqbdata->setId( $row["id"] );				
				$sqbdata->setCourseId( stripslashes($row["course_id"]) );
				$sqbdata->setUserId( stripslashes($row["user_id"]) );				
				$sqbdata->setDate( stripslashes($row["date"]) );				
				$sqbdataReturn = $sqbdata;
		}

		return $sqbdataReturn;
	} 
	
	public static function load() {
		$dap_dbh = Dap_Connection::getConnection(); 
		$sqbdataarray = null;

		//Load product details from database
		$sql = "select * from dap_user_certificate ";
		$stmt = $dap_dbh->prepare($sql);
		$stmt->execute();
		while ($row = $stmt->fetch()) {
				$sqbdata = new Dap_UserCertificate();					
				$sqbdata->setId( $row["id"] );
				$sqbdata->setCourseId( stripslashes($row["course_id"]) );
				$sqbdata->setUserId( stripslashes($row["user_id"]) );
				$sqbdata->setDate( stripslashes($row["date"]) );
				$sqbdataarray[] = $sqbdata;
		}

		return $sqbdataarray;
	}
	
	
	 
	public static function deleteById($id = 0) {
		$dap_dbh = Dap_Connection::getConnection(); 
		$sql = "delete from dap_user_certificate where id = :id";
		$stmt = $dap_dbh->prepare($sql);
		$stmt->bindParam(':id', $id, PDO::PARAM_INT);
		$stmt->execute();

		return $id;
	}
	
	
}
