<?php
class dap_xenapi {
	const PARAMETERS_FIELDS  = 'dap_xenapi:API_KEY:PROTOCOL:API_URL';
	/**
	* Default constructor.
	*/
	function dap_xenapi()
	{ 
	}

	/**
	* This function is called by DAP once a user registers.
	*/
	function register($user_id, $product_id, $parameters) 
	{
		// Tell the log that we recieved the request.
		$this->log(
			'register', 
			'Registering user ID: ' . $user_id . ' with XenAPI.'
		);

		// Grab the DAP user from the user ID.
		$dap_user = Dap_User::loadUserById($user_id);

		// Set the unique username.
		/*$username = trim($dap_user->getFirst_name()) 
				  . ' ' . trim($dap_user->getLast_name());
        */
		
		$username=$dap_user->getUser_name();
		
		if($username=="") {
		  $username=generateUsername("dap_xenapi.class.php.register",$dap_user->getEmail(),$dap_user->getFirst_name(),$dap_user->getLast_name());
		  
		  if($username=="") {
			  $username = trim($dap_user->getFirst_name()) 
					. ' ' . trim($dap_user->getLast_name());
			  logToFile("dap_xenapi.class.php.register: generated username set to NULL. Set username to first+lastname=".$username,LOG_INFO_DAP);
			  $dap_user->setUser_name($username);
			  $dap_user->update();
		  }
		 
		}
		// Tell the log that we recieved the request.
		$this->log(
			'register', 
			'Registering user ID: ' 
				. $user_id . ' with username "' . $username . '".'
		);

		// Set the variables of the DAP user.
		$user_data = array(
			'username'   => $username,
			'password'   => trim($dap_user->getPassword()),
			'email'      => trim($dap_user->getEmail()),
			'user_state' => 'valid',
			'ip_address' => trim($dap_user->getIpaddress())
		);
		
		$email=trim($dap_user->getEmail());
		
		$edit_user_data = array(
			'username'   => $username
		);
		// Split the parameters into an array.
		$data = explode(':', $parameters);

		// Initialize the API data.
		try 
		{
			$api_data = $this->initializeAPIData($data);
		} 
		catch (Exception $e) 
		{
			// The initilization failed, log error and return exception message.
			$this->log('initializeAPIData', $e->getMessage(), 'error');
			return $e->getMessage();
		}

		// Get the additional parameters.
		try 
		{
			$additional_parameters = $this->getAdditionalParameters(
					$data, 
					$parameters
				);
		} 
		catch (Exception $e) 
		{
			// The API query failed, log error and return exception message.
			$this->log('getAdditionalParameters', $e->getMessage(), 'error');
			return $e->getMessage();
		}

		// Check if the group parameter is set.
		if (!empty($additional_parameters['group'])) 
		{
			// Log that we found a custom field identifier.
			$this->log(
				'register', 
				'Found group: ' . $additional_parameters['group']
			);

			// Set which group the user should have.
			$user_data['add_groups'] = $additional_parameters['group'];
			$edit_user_data['add_groups'] = $additional_parameters['group'];
			
		}

		// Check if the custom field identifier parameter is set.
		if (!empty($additional_parameters['custom_field_identifier'])) 
		{
			// Log that we found a custom field identifier.
			$this->log(
				'register', 
				'Found custom field identifier: ' 
					. $additional_parameters['custom_field_identifier'] 
					. '=' . $username . ' ' . $user_id
			);

			// Set the custom field identifier.
			$user_data['custom_fields'] = $additional_parameters['custom_field_identifier'] 
										. '=' 
										. $username . ' ' . $user_id;
										
			$edit_user_data['custom_fields'] = $additional_parameters['custom_field_identifier'] 
										. '=' 
										. $username . ' ' . $user_id;							
		}

		// Unset the additional parameters.
		unset($additional_parameters);

		// Execute the API query.
		try 
		{
			$this->apiRegister(
				$api_data['api_url'], 
				$api_data['api_key'], 
				$user_data
			);
		} 
		catch (Exception $e) 
		{
	
			// The API query failed, log error and return exception message.
			//$this->log('apiRegister', $e->getMessage(), 'error');
			
			//try update
			// Execute the API query.
		
				// User exist, let's edit the user instead.
			$this->log(
				'handleAPIError', 
				'User already exist, attempting to edit user instead.', 
				'warning'
			);
			
			// Init the edit data.
			$edit_data = array(
				'user'       => $user_data['username'],
				'add_groups' => $user_data['add_groups']
			);
			
				
			// Init the edit data.
			$edit_data_email = array(
				'email'       => $user_data['email'],
				'add_groups' => $user_data['add_groups']
			);
			
			
			// Check if the custom field identifier parameter is set.
			if (!empty($user_data['custom_field_identifier'])) 
			{
				// Log that we found a custom field identifier.
				$this->log(
					'handleAPIError', 
					'Found custom field identifier: ' 
						. $user_data['custom_field_identifier']
				);
				// Set the custom field identifier.
				$edit_data['custom_field_identifier'] = $user_data['custom_field_identifier'];
				$edit_data_email['custom_field_identifier'] = $user_data['custom_field_identifier'];
			}
			
			try {
				// Execute the API edit request.
				$this->apiEditUser($api_data['api_url'], $api_data['api_key'], $edit_data);		
				
				/*$this->apiEditUser(
					$api_data['api_url'], 
					$api_data['api_key'], 
					$edit_user_data
				);*/
				
				return 0;
			} 
			catch (Exception $e) 
			{
				// The API query failed, log error and return exception message.
				// try edit using email
				
				try {
					//$xfuser=$this->getUser($api_data['api_url'], $api_data['api_key'], $edit_data_email);		
					//$xfusername=$xfuser["username"];
					
					//$user = $this->xenAPI->getUser($email);
					
					$username=$this->getXFUser(
					  $api_data['api_url'], 
					  $api_data['api_key'], 
					  $email
					);
					
					if($username!=-1) {
						$this->log('getXFUser', "SUCCESS", 'Got username from Xenforo');
						$edit_data["user"]=$username;
						$this->apiEditUser($api_data['api_url'], $api_data['api_key'], $edit_data);	
						return 0;
					}
					
					$this->log('Register', "Could not find the user in Xenforo", 'editerror');
					return $e->getMessage();
					
				}
				catch (Exception $e) 
				{
					$this->log('getResults', $e->getMessage(), 'editerror');
					return $e->getMessage();
				}
				
				$this->log('getResults', $e->getMessage(), 'error');
				return $e->getMessage();
			}
			
			
			return $e->getMessage();
		}

		return 0;
	}



	/**
	* This function is called by DAP once a user is removed.
	*/
	function unregister($user_id, $product_id, $parameters)
	{
		// Tell the log that we recieved the request.
		$this->log(
			'unregister', 
			'Editing user ID: ' . $user_id . ' with XenAPI.'
		);

		// Grab the DAP user from the user ID.
		$dap_user = Dap_User::loadUserById($user_id);

		$username=$dap_user->getUser_name();
		$email=$dap_user->getEmail();
		
		if($username=="") {
			
			// Set the unique username.
			$username = trim($dap_user->getFirst_name()) 
				  . ' ' . trim($dap_user->getLast_name());
		}
		
		// Set the variables of the DAP user.
		$user_data = array(
			'user' => $username
		);
		$user_data_email = array(
			'email' => $email
		);
		
		// Split the parameters into an array.
		$data = explode(':', $parameters);

		// Initialize the API data.
		try 
		{
			$api_data = $this->initializeAPIData($data);
		} 
		catch (Exception $e) 
		{
			// The initilization failed, log error and return exception message.
			$this->log('initializeAPIData', $e->getMessage(), 'error');
			return $e->getMessage();
		}

		// Get the additional parameters.
		try 
		{
			$additional_parameters = $this->getAdditionalParameters(
				$data, 
				$parameters
			);
		} 
		catch (Exception $e) 
		{
			// The API query failed, log error and return exception message.
			$this->log('getAdditionalParameters', $e->getMessage(), 'error');
			return $e->getMessage();
		}

		// Check if the group parameter is set.
		if (!empty($additional_parameters['group'])) 
		{
			// Log that we found a custom field identifier.
			$this->log(
				'unregister', 
				'Found group: ' . $additional_parameters['group']
			);

			// Set which group to remove.
			$user_data['remove_groups'] = $additional_parameters['group'];
			//$user_data['add_groups'] = "1";
			$user_data_email['remove_groups'] = $additional_parameters['group'];
			//$user_data_email['add_groups'] = "1";
		}

		// Check if the custom field identifier parameter is set.
		if (!empty($additional_parameters['custom_field_identifier'])) 
		{
			// Log that we found a custom field identifier.
			$this->log(
				'unregister', 
				'Found custom field identifier: ' 
					. $additional_parameters['custom_field_identifier']
			);

			// Set the custom field identifier.
			$user_data['custom_field_identifier'] = $additional_parameters['custom_field_identifier'];
			$user_data_email['custom_field_identifier'] = $additional_parameters['custom_field_identifier'];
		}
		
		
		try {
			$username=$this->getXFUser(
					  $api_data['api_url'], 
					  $api_data['api_key'], 
					  $username
			);
			
			if($username!=-1) {
				
				$this->apiEditUser(
					$api_data['api_url'], 
					$api_data['api_key'], 
					$user_data
				);
				
				logToFile( "unregister: username is in sync. Completed update successfully");
				
				unset($additional_parameters);
				return 0;
			}
			
			$user=$this->getXFUserFull(
						  $api_data['api_url'], 
						  $api_data['api_key'], 
						  $email
			);
			
			if($user!=-1) {
			
			//	$user["remove_groups"]=$additional_parameters['group'];
				$remove_groups=$this->editUser(
					$api_data['api_url'], 
					$api_data['api_key'], 
					$user, 
					$additional_parameters['group']
				);
			
				if($remove_groups!=-1) {
					logToFile( "unregister: username not in sync. Completed remove_group call successfully");
					return 0;	
				}
				/*$this->apiEditUser(
					$api_data['api_url'], 
					$api_data['api_key'], 
					$user_data
				);*/
				
				unset($additional_parameters);
				logToFile( "unregister: username not in sync. Completed update successfully");
				return 0;
			}
			
			unset($additional_parameters);
			$this->log('getResults', "could not get user details", 'error');
			return $e->getMessage();
			
		}
		catch (Exception $e) 
		{
			// Unset the additional parameters.
			unset($additional_parameters);
			// The API query failed, log error and return exception message.
			$this->log('getResults', $e->getMessage(), 'error');
			return $e->getMessage();
		}
		
		return 0;
	}
	
	
	private function getXFUser($post_url, $api_key, $email) {
	  $post_data = array('hash'   => $api_key,
						 'action' => 'getUser',
						 'value'  => $email);
	  
	  				 
	  $handle = curl_init();
	  curl_setopt($handle, CURLOPT_URL, $post_url);
	  curl_setopt($handle, CURLOPT_RETURNTRANSFER, 1);
	  curl_setopt($handle, CURLOPT_POST, 1);
	  curl_setopt($handle, CURLOPT_POSTFIELDS, $post_data);
	  
	  $output = curl_exec($handle);
	  $http_status_code = curl_getinfo($handle, CURLINFO_HTTP_CODE);
	  curl_close($handle);
	  $json_decode = json_decode($output, TRUE);
	  //header ('Content-type: text/plain');
	  
	  if ($http_status_code == 200) {
	//	  print_r($json_decode);
	//	logToFile( "getXFUser() : success... usename=".$json_decode["username"]);
		$username=$json_decode["username"];
		$this->log('getXFUser', "success: username=".$username, 'success');
		
		return $username;
	  } else {
		  //echo 'ERROR! Status code is not 200 (' . $http_status_code . ')\n';
		 // logToFile( "getXFUser() : error... could not get user details");
		  $this->log('getXFUser', "ERROR: could not get user details", 'error');
		  return -1;
		//  print_r($json_decode);
	  }
	  
	  
	  
	}
	
	private function editUser($post_url, $api_key, $user, $groupId) {
	  $post_data = array('hash'   => $api_key,
						 'action' => 'editUser',
						 'user'  => $user["username"],
						 'remove_groups' => $groupId);
	  
	  $this->log('getXFUser', "remove from group = " . $groupId, 'editUser-removegroup');				 
	  $this->log('getXFUser', "username = " . $user["username"], 'editUser-removegroup');		
	  $this->log('getXFUser', "current groups = " . $user["secondary_group_ids"], 'editUser-removegroup');			 
	  $handle = curl_init();
	  curl_setopt($handle, CURLOPT_URL, $post_url);
	  curl_setopt($handle, CURLOPT_RETURNTRANSFER, 1);
	  curl_setopt($handle, CURLOPT_POST, 1);
	  curl_setopt($handle, CURLOPT_POSTFIELDS, $post_data);
	  
	  $output = curl_exec($handle);
	  $http_status_code = curl_getinfo($handle, CURLINFO_HTTP_CODE);
	  curl_close($handle);
	  $json_decode = json_decode($output, TRUE);
	  //header ('Content-type: text/plain');
	  
	  if ($http_status_code == 200) {
	//	  print_r($json_decode);
	//	logToFile( "getXFUser() : success... usename=".$json_decode["username"]);
		$remove_groups=$json_decode["remove_groups"];
		$this->log('editUser', "success: remove_groups=".$remove_groups, 'success');
		
		return $remove_groups;
	  } else {
		  //echo 'ERROR! Status code is not 200 (' . $http_status_code . ')\n';
		 // logToFile( "getXFUser() : error... could not get user details");
		  $this->log('editUser', "ERROR: could not remove group", 'error');
		  return -1;
		//  print_r($json_decode);
	  }
	  
	  
	  
	}

	private function getXFUserFull($post_url, $api_key, $email) {
	  $post_data = array('hash'   => $api_key,
						 'action' => 'getUser',
						 'value'  => $email);
	  
	  				 
	  $handle = curl_init();
	  curl_setopt($handle, CURLOPT_URL, $post_url);
	  curl_setopt($handle, CURLOPT_RETURNTRANSFER, 1);
	  curl_setopt($handle, CURLOPT_POST, 1);
	  curl_setopt($handle, CURLOPT_POSTFIELDS, $post_data);
	  
	  $output = curl_exec($handle);
	  $http_status_code = curl_getinfo($handle, CURLINFO_HTTP_CODE);
	  curl_close($handle);
	  $json_decode = json_decode($output, TRUE);
	  //header ('Content-type: text/plain');
	  
	  if ($http_status_code == 200) {
	//	  print_r($json_decode);
	//	logToFile( "getXFUser() : success... usename=".$json_decode["username"]);
		$username=$json_decode["username"];
		$this->log('getXFUser', "success: username=".$username, 'success');
		
		return  $json_decode;
	  } else {
		  //echo 'ERROR! Status code is not 200 (' . $http_status_code . ')\n';
		 // logToFile( "getXFUser() : error... could not get user details");
		  $this->log('getXFUser', "ERROR: could not get user details", 'error');
		  return -1;
		//  print_r($json_decode);
	  }
	  
	  
	  
	}


	private function handleAPIError($api_response, $api_url, $api_key, $action, 
		$user_data
	) {
		// Decode the JSON.
		$api_response = json_decode($api_response, TRUE);

		/* 
		Check if the JSON decode failed 
		(meaning that the results was not a JSON string).
		*/
		if (function_exists('json_last_error'))
		{
			// PHP version >= 5.3.0 required for json_last_error().
			if (json_last_error() != JSON_ERROR_NONE) 
	 		{
				// The request failed, throw exception.
				$this->log(
					'handleAPIError', 
					'Request failed, results was not a JSON string: ' 
						. $api_response,
		  			'error'
				);
				return NULL;
	 		}
		}
		else if ($api_response === NULL) 
		{
			// PHP < 5.3.0 workaround for json_last_error().
			// The request failed, throw exception.
			$this->log(
				'handleAPIError', 
				'Request failed, results was not a JSON string: ' 
					. $api_response,
	  			'error'
			);
			return NULL;
		}

		// Log the error ID and error message.
		$this->log(
			'handleAPIError', 
			'API returned error id ' . $api_response['error'] . ': ' 
				. $api_response['message'], 
			'error'
		);

		// Check if the error is a user error.
		if (!empty($api_response['user_error_id']))
		{
			/* 
			The error was a user error, 
			log the user error ID and user error message/phrase.
			*/
			$this->log(
				'handleAPIError', 
				'Found user error id ' . $api_response['user_error_id'] . ': ' 
					. $api_response['user_error_phrase'], 
				'error'
			);

			// Check if user already exist.
			if ($api_response['user_error_id'] == 40 
				&& !empty($user_data['add_groups'])
			) {
				// User exist, let's edit the user instead.
				$this->log(
					'handleAPIError', 
					'User already exist, attempting to edit user instead.', 
					'warning'
				);

				// Init the edit data.
				$edit_data = array(
					'user'       => $user_data['username'],
					'add_groups' => $user_data['add_groups']
				);

				// Check if the custom field identifier parameter is set.
				if (!empty($user_data['custom_field_identifier'])) 
				{
					// Log that we found a custom field identifier.
					$this->log(
						'handleAPIError', 
						'Found custom field identifier: ' 
							. $user_data['custom_field_identifier']
					);

					// Set the custom field identifier.
					$edit_data['custom_field_identifier'] = $user_data['custom_field_identifier'];
				}
				// Execute the API edit request.
				$this->apiEditUser($api_url, $api_key, $edit_data);			}

			// Check if the user error field is set.
			if (!empty($api_response['user_error_field']))
			{
				// User error field is set, log it.
				$this->log(
					'handleAPIError', 
					'User error field: ' . $api_response['user_error_field'], 
					'error'
				);
			}

			// Check if the user error key is set.
			if (!empty($api_response['user_error_key']))
			{
				// User error key is set, log it.
				$this->log(
					'handleAPIError', 
					'User error key: ' . $api_response['user_error_key'], 
					'error'
				);
			}
		}
	}

	private function getAdditionalParameters($data, $parameters)
	{
		// Init the additional parameters.
		$additional_parameters = array();

		// Check if group parameter is set.
		if (isset($data[4]))
		{
			if (empty($data[4]) && !is_numeric($data[4]))
			{
		 		// Group is set but empty, throw error message.
				throw Exception(
					'Group ID is empty. Params should be ' 
				  	. '(' . self::PARAMETERS_FIELDS . ':GROUP), but is (' 
			  		. $parameters . ')'
 				);
			}

			// Check if group ID is numeric and larger than 0 (we ignore 0).
			if (is_numeric($data[4]) && $data[4] > 0)
			{
				// Set the group.
				$additional_parameters['group'] = $data[4];
			}
		}

		// Check if the custom identifier field parameter is set.
		if (isset($data[5]))
		{
			if (empty($data[5]))
			{
		 		/* 
		 		Custom identifier field is set but empty, throw error message.
		 		*/
				throw Exception(
					'Custom identifier field is set but empty. Params should be' 
 				  	. ' (' . self::PARAMETERS_FIELDS 
			  		. ':GROUP:CUSTOM_USER_FIELD), but is (' . $parameters . ')'
 				);
			}

			/*
			Set the custom identifier field of 
			which we want to identify the user with.
			*/
			$additional_parameters['custom_field_identifier'] = $data[5];
		}
		return $additional_parameters;
	}

	private function initializeAPIData($data) 
	{
		// Grab the API key from the parameters.
		$api_key = $this->getAPIKey($data);

		// Check if we found the API key from the parameters.
		if ($api_key == NULL) 
		{
			// Could not find an API key, throw exception.
			throw new Exception(
				'Missing API key. Params should be ' 
		  		. '(' . self::PARAMETERS_FIELDS 
		  		. '), but is (' . $parameters . ')'
			);
		}

		// We assume the API key was found.
		$this->log('initializeAPIData', 'Found API key = ' . $api_key . '.');

		// Grab the API key from the parameters.
		$api_url = $this->getAPIURL($data);

		// Check if we found the API URL from the parameters.
		if ($api_url == NULL) 
		{
			// Could not find an API URL, throw exception.
			throw new Exception(
				'Missing API URL. Params should be ' 
		  		. '(' . self::PARAMETERS_FIELDS . '), but is (' 
	  			. $parameters . ')'
			);
		}

		// We assume the API URL was found.
		$this->log('initializeAPIData', 'Found API URL = ' . $api_url);

		return array('api_key' => $api_key, 'api_url' => $api_url);
	}

	/**
	* Get the API key from the intput parameters, 
	* returns NULL if no API key could be found.
	*/
	private function getAPIKey($data) 
	{
		if (isset($data[1]) && !empty($data[1])) 
		{
			// We assume the API key was found and return it.
			return $data[1];
		}
		// An API key could not be found, returning NULL.
		return NULL;
	}

	/**
	* Get the API url from the intput parameters, 
	* returns NULL if no API url could be found.
	*/
	private function getAPIURL($data) 
	{
		if (isset($data[2]) && !empty($data[2]) 
			&& isset($data[3]) && !empty($data[3])) 
		{
			// Lowercase the protocol.
			$data[2] = strtolower($data[2]);

			// Let's make sure the protocol is correct.
			if ($data[2] != 'http' && $data[2] != 'https')
			{
				// The protocol was invalid, log error and return NULL.
				$this->log(
					'getAPIURL', 
					'The protocol was invalid, expected "http" or "https",'
						. 'but got: ' . $data[2], 
					'error'
				);
				return NULL;
			}

			// We assume the URL key was found and return it.
			return strtolower($data[2]) . '://' . $data[3];
		}
		// An API URL could not be found, returning NULL.
		return NULL;
	}

	/**
	* Make it easier to log the messages to file.
	*/
	protected function log($method, $message, $log_level = '')
	{
		// Set the log prefix.
		$log_prefix = 'dap_xenapi.class.php: ' . $method . '(): ' 
					. (!empty($log_level) ? strtoupper($log_level) . ': ' : '')
					. ': ';

		/*
		Split the message into an array as 
		DAP only allows the max length of 200 characters.
		*/
		$message_array = str_split($message, 200 - strlen($log_prefix));

		// Loop through the messages and log them.
		foreach ($message_array as $message) 
		{
			// Log the message.
			logToFile($log_prefix . $message, LOG_INFO_DAP);
		}
	}

	private function apiRegister($api_url, $api_key, $user_data) 
	{
		$this->getResults($api_url, $api_key, 'register', $user_data);
	}

	private function apiEditUser($api_url, $api_key, $user_data) 
	{
		$this->getResults($api_url, $api_key, 'editUser', $user_data);
	}
	
	private function getUser($api_url, $api_key, $user_data) 
	{
		$user = $this->getResults($api_url, $api_key, 'getUser', $user_data);
		return $user;
	}

	/**
	* This function handles everything related to the XenAPI, 
	* The function will fallback to file_get_contents if cURL is not found.
	* it will either return a JSON string, or NULL if something wrong happend.
	*/
	private function getResults($api_url = NULL, $api_key = NULL, 
		$action = NULL, array $user_data
	) {
		// Check if all the required parameters have been initialized.
		if ($api_url == NULL || $api_key == NULL || $action == NULL)
		{
			// One of the variables were not initialized, throe exception.
			throw new Exception(
				'One or more of the required parameters were NULL. ' 
				. 'api_key=' . $api_url . ', api_key=' . $api_key 
				. ', action=' . $action
		   );
		}

		// Initialize the API data.
		$api_data = array(
			'hash' => $api_key,
			'action' => $action
		);

		// Merge the API data array with the user data array.
		$api_data = array_merge($user_data, $api_data);

		// Check if cURL is available, fallback to file_get_contents if not.
		if (is_callable('curl_init'))
		{
			// cURL was found avaiable.
			$this->log('getResults', 'cURL was found.');

			// Initialize cURL with the input values.
			$curl_handle = curl_init();
			curl_setopt($curl_handle, CURLOPT_URL, $api_url);
			curl_setopt($curl_handle, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl_handle, CURLOPT_POST, 1);
			curl_setopt($curl_handle, CURLOPT_POSTFIELDS, $api_data);

			// Grab thet data of the cURL data.
			$response = curl_exec($curl_handle);

			// Check if something went wrong with the request.
			if ($response === FALSE)
			{
				// The cURL request failed, throw exception.
				throw new Exception(
					'Request failed with action: ' . $action 
				  . ', cURL error: ' . curl_error($curl_handle)
				);
			}

			// Get the HTTP status code.
			$http_status_code = curl_getinfo($curl_handle, CURLINFO_HTTP_CODE);

			// Close th cURL handle.
			curl_close($curl_handle);

			// Check if the status code was 200 (OK).
			if ($http_status_code != 200) 
			{
				// Check if the error parameter is set.
				if (!empty($response['error'])) 
				{
					$this->handleAPIError(
						$response, 
						$api_url, 
						$api_key, 
						$action, 
						$user_data
					);
				}

				/* 
				The request failed, response header did not return status 200, 
				throw exception.
				*/
				throw new Exception(
					'Request failed with action: ' . $action 
				  . ', HTTP status code was not 200: ' . $http_status_code
				);
			}
		}
		else
		{
			// cURL is not avaiable, fallback to file_get_contents instead.
			$this->log(
				'getResults', 
				'Could not find cURL, '
					.'falling back to file_get_contents instead', 
				'warning'
			);

			// Options for the stream.
			$options = array(
			    'http' => array(
			        'header'  => "Content-type: " 
			        		   . "application/x-www-form-urlencoded\r\n",
			        'method'  => 'POST',
			        'content' => http_build_query($api_data),
			    ),
			);

			// Create a stream with the options we set above.
			$context  = stream_context_create($options);

			// Get the results of the API query.
			$response = file_get_contents($api_url, FALSE, $context);

			// Check if something went wrong with the request.
			if ($response === FALSE) 
			{
				// The request failed, throw exception.
				throw new Exception('Request failed with action: ' . $action);
			}

			// Check if the status code was 200 (OK).
			if (strpos($http_response_header[0], '200') === FALSE) 
			{
				// Check if the error parameter is set.
				if (!empty($response['error'])) 
				{
					$this->handleAPIError(
						$response, 
						$api_url, 
						$api_key, 
						$action, 
						$user_data
					);
				}

				/* 
				The request failed, 
				response header did not return status 200, throw exception.
				*/
				throw new Exception(
					'Request failed with action: ' . $action 
				  	. ', HTTP status code was not 200: ' 
				  	. $http_response_header[0]
				);
			}
		}
		/* 
		For debugging: 
		$this->log(
			'getResults', 
			'Got response from the XenAPI:' . $response . '.'
		);
		*/

		// Decode the JSON.
		$response_decoded = json_decode($response, TRUE);

		/* 
		Check if the JSON decode failed 
		(meaning that the results was not a JSON string).
 		*/
		if (function_exists('json_last_error'))
		{
			// PHP version >= 5.3.0 required for json_last_error().
			if (json_last_error() != JSON_ERROR_NONE) 
	 		{
				// The request failed, throw exception.
				throw new Exception(
					'Request failed with action: ' . $action 
				  . ', results was not a JSON string: ' . $response
				);
	 		}
		}
		else if ($response_decoded === NULL) 
		{
			// PHP < 5.3.0 workaround for json_last_error().
			// The request failed, throw exception.
			throw new Exception(
				'Request failed with action: ' . $action 
			  . ', results was not a JSON string: ' . $response
			);
		}

 		/* 
 		The results was a JSON response and contained no errors, 
 		return the results.
 		*/
 		return $response_decoded;
	}
}




?>
